/*
 * Copyright (c) 2002-2013, Mairie de Paris
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *  1. Redistributions of source code must retain the above copyright notice
 *     and the following disclaimer.
 *
 *  2. Redistributions in binary form must reproduce the above copyright notice
 *     and the following disclaimer in the documentation and/or other materials
 *     provided with the distribution.
 *
 *  3. Neither the name of 'Mairie de Paris' nor 'Lutece' nor the names of its
 *     contributors may be used to endorse or promote products derived from
 *     this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * License 1.0
 */
package fr.paris.lutece.plugins.form.business.file;

import fr.paris.lutece.portal.service.plugin.Plugin;
import fr.paris.lutece.portal.service.spring.SpringContextService;


/**
 * This class provides instances management methods (create, find, ...) for file objects
 */
public final class FileHome
{
    private static final String BEAN_FORM_FILE_DAO = "form.fileDAO";
    private static IFileDAO _dao = SpringContextService.getBean( BEAN_FORM_FILE_DAO );

    /**
     * Private constructor - this class need not be instantiated
     */
    private FileHome(  )
    {
    }

    /**
     * Creation of an instance of record file
     * @param file The instance of the file which contains the informations to store
     * @param plugin the plugin
     * @return the id of the file after creation
     */
    public static int create( File file, Plugin plugin )
    {
        return _dao.insert( file, plugin );
    }

    /**
     * Update of file which is specified in parameter
     * @param  file The instance of the  record file which contains the informations to update
     * @param plugin the plugin
     */
    public static void update( File file, Plugin plugin )
    {
        _dao.store( file, plugin );
    }

    /**
     * Delete the file whose identifier is specified in parameter
     * @param nIdFile The identifier of the record file
     * @param plugin the plugin
     */
    public static void remove( int nIdFile, Plugin plugin )
    {
        _dao.delete( nIdFile, plugin );
    }

    /**
     * Returns an instance of a file whose identifier is specified in parameter
     * @param nKey The file  primary key
     * @param plugin the plugin
     * @return an instance of file
     */
    public static File findByPrimaryKey( int nKey, Plugin plugin )
    {
        return _dao.load( nKey, plugin );
    }
}
